@@ -1,9 +1,9 @@
 #!/usr/bin/perl
 
 use strict;
+use warnings;
 use lib qw(blib/lib blib/arch);
 use Cv;
-use Scalar::Util qw(blessed);
 
 my $capture;
 my $videoSource;
